function wall_clock_time = check_ICS_runtime(modelparams,numpoints, numprocs, a_att)

% wall_clock_time = check_ICS_runtime(modelparams,numpoints, numprocs, a_att)
% Check the computation time necessary for the current configuration
% The returned time estimate is a rough estimate based on a simple linear
% extrapolation of the computation time needed for a specific lossless reference 
% problem on the Huygens computer. This routine requires some more sophistication...
% Inputs:
% modelparams:  structure that has been generated with set_ICS_model.
% numpoints:	11x4 element array containing the number of points in the
%               T/X/Y/Z dimensions, as is being generated by check_ICS_memory.
% numprocs:     number of processors.
% a_att:        (optional, default = 0) Frequency-dependent attenuation 
%               parameter [Np/cm MHz^b]
% Output:
% wall_clock_time: wall clock time in hours.

% JH - version 090126

if(nargin<3)
    error('Not enough arguments')
elseif(nargin<4 || isempty(a_att))
    a_att = 0;
end
if(nargin>4)
    error('Too many arguments')
end

%check wall clock time on the basis of the 
% wall clock times as were needed in the SHI/10MHZ_102_newtx simulation
% assume a linear relation with the total number of points
numtimerefpoints=prod([575  300   60  450]);
lf = 594.67-1.13;       %primary field solution
nf = 2390.67-594.67;    %field iteration 
b2b= 13535.46-11876.97; %beam-to-beam solution
s2b = lf*2;         %single-plane-to-beam solution (=lf sol + z diff), rough estimate

% Linear correction factor for the number of points; multiply by 4 for
% losses - rough estimate
if(a_att==0)
    timefactor = prod(numpoints(1,:))/numtimerefpoints;
else
    timefactor = prod(numpoints(1,:))/numtimerefpoints*4;
end

numbeams=numel(modelparams.beamiterations);

wall_clock_time= numbeams*lf + sum(modelparams.beamiterations)*nf;

if(numbeams>1)
    wall_clock_time = wall_clock_time+(numbeams-1)*b2b;
end
if(numbeams>2)
    wall_clock_time = wall_clock_time+(numbeams-2)*s2b;
end

wall_clock_time = wall_clock_time *timefactor/3600;
