function sourceparams = set_ICS_source_parametric(aperture,signature,...
    excitationtype)

% sourceparams = set_ICS_source_parametric(aperture,signature,excitationtype)
% Generates the sourceparams structure that contains all source settings
% used in the ICS Parnac program for a parametric source.
% Inputs:
% aperture:        aperture structure generated by one of the
%                  set_ICS_aperture_ functions
% signature:       signature structure generated by one of the 
%                  set_ICS_signature_ functions
% excitationtype:  (optional, default = 'fsource') Type of source excitation,
%                  choose between 'qsource','fsource','dtqsource' or 'planewave'

% JH, version 090116

if(nargin<2)
    error('Not enough arguments')
end
if(nargin<3 || isempty(excitationtype))
    excitationtype = 'fsource';
end
if(nargin>3)
    error('Too many arguments')
end

if(isempty(strmatch(excitationtype,strvcat(...
        'qsource','fsource','dtqsource','planewave'), 'exact')))
    error('excitationtype has an invalid value')
end

sourceparams = struct('excitationtype',excitationtype,...
    'definitiontype','parametric',...    
    'aperture',aperture,'signature',signature);
